#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <unistd.h>
#include <getopt.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "debug.h"
#include "convert.h"
#include "code.h"
#include "binding.h"

static int
add_code_full_var(gchar *fullvar)
{
	gchar *left, *right, *pos;
	int len;

	pos = strstr(fullvar, "=");

	if (!pos) {
		errormsg("Could not find `=' in `%s'", fullvar);
		return -1;
	}

	len = pos - fullvar;

	left = malloc(len + 1);
	strncpy(left, fullvar, len);
	left[len] = '\0';

	right = malloc(strlen(fullvar) - len);
	strcpy(right, &fullvar[len + 1]);

	variable_add(left, right);

	free(left);
	free(right);

	return 0;
}

static int
add_builtin_variables(void)
{
	struct tm *tm;
	time_t t;
	gchar buffer[26];

	time(&t);
	tm = gmtime(&t);

	strftime(buffer, 25, "%Y-%0m-%0d", tm);
	variable_add("today", buffer);
	return 0;
}

static xmlDocPtr
parse_xml_doc(const gchar * xmlfilename, gboolean validate)
{
    xmlParserCtxtPtr ctxt; /* Pointer to the parser context */
    xmlDocPtr doc;

    /* Init libxml */     
    xmlInitParser();
    LIBXML_TEST_VERSION

    if (validate) {
	/* create a parser context */
	ctxt = xmlNewParserCtxt();
	if (ctxt == NULL) {
	    fprintf(stderr, "Failed to allocate parser context\n");
	    return NULL;
	}
		
	doc = xmlCtxtReadFile(ctxt, xmlfilename, NULL,
			      XML_PARSE_DTDATTR |  /* default DTD attr's */
			      XML_PARSE_NOENT |    /* subst. entities */
			      XML_PARSE_DTDVALID); /* validate with DTD */
		
	/* check if validation succeeded */
	if (ctxt->valid == 0) {
	    errormsg("Failed to validate %s.\n", xmlfilename);
	    xmlFreeParserCtxt(ctxt);
	    xmlFreeDoc(doc);
	    return NULL;
	}

	if (!doc) {
	    fprintf(stderr, "Error parsing xml file %s. \n", xmlfilename);
	    xmlFreeParserCtxt(ctxt);
	    xmlFreeDoc(doc);
	    return NULL;
	}
    }
    else {
	doc = xmlReadFile(xmlfilename, NULL, 0);
	if (!doc) {
	    fprintf(stderr, "Error parsing xml file %s. \n", xmlfilename);
	    xmlFreeDoc(doc);
	    return NULL;
	}
    }
    return doc;
}
	

int
main(int argc, gchar *argv[])
{
	xmlDocPtr doc;

	gboolean validate = FALSE;
     
	gint errflg = 0;
	gchar c;
	extern gchar *optarg;
	extern gint optind, opterr, optopt;

	debug_init();

	while ((c = getopt(argc, argv, "a:dh")) != -1) {
		switch(c) {
		case 'a':
			if (optarg)
				if (add_code_full_var(optarg) < 0)
					errflg++;
			break;
		case ':':
			fprintf(stderr,
				"Option -%c requires an operand\n", 
				optopt);
			errflg++;
			break;
		case 'd':
			validate = TRUE;
			break;
		case 'h':
			errflg++;
			break;
		case '?':
			fprintf(stderr,
				"Unrecognized option: -%c\n", 
				optopt);
			errflg++;
			break;
		}
	}

/* Also the long options:
`--params could
c = getopt_long(argc, argv, longoptions)


	if (errflg) {
		fprintf(stderr, 
			"Usage: %s [-a <argument>=<value>] [-v] <inputfile>"
			  "\n\t-a\tParse an argument to the backend"
			"\n\t-d\tValidate the document (off by default)\n", 
			argv[0]);
		return EXIT_FAILURE;
	}


	
	add_builtin_variables();
	

	/* code_init() not implemented */
	if (code_init() == -1) {
		errormsg("Unable to initialize code library");
		return EXIT_FAILURE;
	}
	
	/* Parse the XML document, validate if option was selected */
	if ((doc = parse_xml_doc(argv[optind], validate)) < 0) {
		fprintf(stderr, "Error parsing xml file %s. \n", argv[optind]);
		return EXIT_FAILURE;
	}

	/* Initialise binding libraries */
#ifdef WIN32
   	
#else
        binding_init(doc);
#endif	
	convert_report(doc, stdout);

	/* code_cleanup() not really implemented */
	code_cleanup();
	
	debug_cleanup();
	
	return errors_occured == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
}
