#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "debug.h"
#include "datasource.h"
#include "xml_funcs.h"

#ifdef WIN32
#include "binding_pg.h"
#endif

typedef struct
	{
	char *name;
	char *value;
	}
Variable;

static GList *Variables = NULL;

char *
variable_get(char *variable)
	{
	GList *cur;
	Variable *v;

	for (cur = Variables; cur != NULL; cur = cur->next)
		{
		v = cur->data;
		if (strcmp(v->name, variable) == 0)
			return v->value;
		}

	/* errormsg ("Could not find variable: %s", variable); */

	return NULL;
	}

int
variable_add(char *variable, char *value)
	{
	Variable *v;

	if (!variable)
		{
		errormsg("variable not set");
		return -1;
		}

	if (!value)
		{
		errormsg("value not set for %s", variable);
		return -1;
		}

	v = malloc(sizeof * v);

	v->name = strdup(variable);
	v->value = strdup(value);

	/* errormsg ("Added variable: '%s' = '%s'", variable, value); */

	Variables = g_list_append(Variables, v);

	return 0;
	}

static char *
variable_process(DataSource * data, char *field)
	{
	char *retval = NULL, *trimtext;

	/* debugmsg ("variable_process: %s, %s", data->query, field); */

	if (!data)
		{
		errormsg("No Datasource Supplied");
		return NULL;
		}

	if (!data->db_query)
		datasource_query(data);
	
#ifndef WIN32

	trimtext = data->db->binding->pap_query_get_value(data->db_query, field);
#else

	trimtext = pap_query_get_value(data->db_query, field);
#endif
	/* retval = trim (trimtext); */
	if (trimtext)
		retval = strdup(trimtext);
	else
		retval = NULL;

	return retval;
	}

char *
variable_process_full(DataSource * data, char *string)
	{
	char *retval;

#if 0

	char *pos;
	char buffer[100];
	int len;
#endif

	if ((retval = variable_get(string)) != NULL)
		return strdup(retval);
	retval = NULL;
	/* 'commented' out */
#if 0

	if ((pos = strchr(string, '.')) == NULL)
		{
		if (data)
			return variable_process(data->name, string);
		else
			{
			errormsg
			("Request for relative name `%s' with no datasource defined",
			 string);
			return NULL;
			}
		}
	else
		{
		len = pos - string;
		strncpy(buffer, string, len);
		buffer[len] = '\0';

		return variable_process(buffer, &string[len + 1]);
		}
#else
	if (data)
		return variable_process(data, string);
	else
		{
		errormsg("Request for non-variable `%s' with no datasource defined",
		         string);
		return NULL;
		}
#endif

	}

int
code_init(void)
	{
	return 0;
	}

int
code_cleanup(void)
	{
	/* Shutdown libxml */
	xmlCleanupParser();

	return 0;
	}
