#ifndef __debug_h
#define __debug_h

#include <stdio.h>
#include <glib.h>
#include <assert.h>

/* some functions */
void debug_printinfo(gchar *file, gint line, gchar *function, void*);
void setdebuglinenum(gint d);
void setdebuginfo(gint d);
/* to show debugging info or not.hmm */
void hidedebuginfo();
void showdebuginfo();
void debug_init();
void debug_cleanup();

FILE * debug_getstream();

extern int errors_occured;

/* Use the following commands as you would use printf(); expect that it provides
	its own end of line formatting */

/* high level error message to stderr */
#define errormsg(fmt,args...) \
do {fprintf(stderr, "Error %s: %d: %s(): " fmt "\n", __FILE__, __LINE__, \
__PRETTY_FUNCTION__ , ##args) ; errors_occured++; } while(0)

/* warning at run time macro. to stderr as well */
#define warningmsg(fmt,args...) \
fprintf(stderr, "Warning %s: %d: %s(): " fmt "\n", __FILE__, __LINE__, \
__PRETTY_FUNCTION__ , ##args)

/* simple quick test debugging functin. This debugging function should no remain
	in your code and you should remove it after using, as it can't be redirected
	to log files etc as debugmsg can be. */
#define message(fmt,args...) \
fprintf(stdout, "<Message: %s: %d: %s()>: " fmt "\n", __FILE__, __LINE__, \
__PRETTY_FUNCTION__ , ##args)

/* proper debugging information file. This will redirect output to a file
	when required or specified. So well tested code goes to a file, and developmental
	code goes to stdout.  Also line numbers are not run in test suite. Use this
	for all standard debugging and printing information about programme running */
#define debugmsg(fmt,args...) \
	debug_printinfo(__FILE__, __LINE__, __PRETTY_FUNCTION__, \
	(void*)fprintf(debug_stream,"" fmt "",  ##args))
/*#define debugmsg(fmt,args...) */

#define dassert(a) \
		do { if (! (a)) { errormsg ("Assertion failure: " #a); exit(1);} } while (0)

extern FILE *debug_stream;

#endif
