#include <stdio.h>
#include <stdlib.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "xml_display.h"

static int xml_display_node_pad (xmlNode *node, int depth);

static int xml_display_attribute_pad (xmlAttr *attribute, int depth)
{
	xmlNode *child;

	if (!attribute)
		return -1;

	printf ("%*sAttribute: %s\n", depth, "", attribute->name);

	for (child = attribute->children; child != NULL; child = child->next)
		{
		xml_display_node_pad (child, depth + 1);
		}

	return 0;
}

int xml_display_attribute (xmlAttr *a)
{
	return xml_display_attribute_pad (a, 0);
}

static int xml_display_node_pad (xmlNode *node, int depth)
{
	xmlNode *child;
	xmlAttr *attr;

	if (!node)
		return -1;

	printf ("%*sNode: %s\n", depth, "", node->name);

	if (node->content)
		printf ("%*sContent: %s\n", depth, "", node->content);

	for (attr = node->properties; attr != NULL; attr = attr->next)
		xml_display_attribute_pad (attr, depth + 1);

	for (child = node->children; child != NULL; child = child->next)
		xml_display_node_pad (child, depth + 1);

	return 0;
}


int xml_display_node (xmlNode *node)
{
	return xml_display_node_pad (node, 0);
}

static int xml_display_doc_pad (xmlDoc *doc, int depth)
{
	xmlNode *node;

	if (!doc)
		return -1;

	printf ("%*sDoc: %s\n", depth, "", doc->name);

	for (node = doc->children ; node != NULL; node = node->next)
		{
		xml_display_node_pad (node, depth + 1);
		}

	return 0;
}

int xml_display_doc (xmlDoc *doc)
{
	return xml_display_doc_pad (doc, 0);
}
