#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <string.h>

#include "equation.h"

GList *functions = NULL;
extern char *eqntext;

int eqnparse ();
int eqn_scan_string (char *);
int eqn_switch_to_buffer (int);

long double equation_retval;

typedef struct 
{
	char *name;
	int nargs;
	union
		{
		func_1 f1;
		func_2 f2;
		func_3 f3;
		} d;
} Function;

int eqnerror (char *s)
{
	fprintf (stderr, "Error at `%s': %s\n", eqntext, s);

	return 0;
}

static void *find_func (char *function, int nargs)
{
	GList *cur;
	Function *f;

	for (cur = functions; cur != NULL; cur = cur->next)
		{
		f = cur->data;
		if (f && f->name && f->nargs == nargs && strcmp (f->name, function) == 0)
			return f;
		}

	return NULL;
}

long double apply_func_1 (char *function, long double arg)
{
	Function *f = find_func (function, 1);
	if (!f)
		return 0;

	return f->d.f1(arg);
}

long double apply_func_2 (char *function, long double arg1, long double arg2)
{
	Function *f = find_func (function, 2);
	if (!f)
		return 0;

	return f->d.f2(arg1, arg2);
}

long double apply_func_3 (char *function, long double arg1, long double arg2, long double arg3)
{
	Function *f = find_func (function, 3);
	if (!f)
		return 0;

	return f->d.f3(arg1, arg2, arg3);
}

int add_func_1 (char *name, func_1 f)
{
	Function *func;

	func = malloc (sizeof *func);
	func->name = strdup (name);
	func->nargs = 1;
	func->d.f1 = f;

	functions = g_list_append (functions, func);

	return 0;
}

int add_func_2 (char *name, func_2 f)
{
	Function *func;

	func = malloc (sizeof *func);
	func->name = strdup (name);
	func->nargs = 2;
	func->d.f2 = f;

	functions = g_list_append (functions, func);

	return 0;
}

int add_func_3 (char *name, func_3 f)
{
	Function *func;

	func = malloc (sizeof *func);
	func->name = strdup (name);
	func->nargs = 3;
	func->d.f3 = f;

	functions = g_list_append (functions, func);

	return 0;
}

long double equation_evaluate (char *equation)
{
	int i;

   eqn_switch_to_buffer (eqn_scan_string (equation));

   i = eqnparse ();

   if (i == 0)
		return equation_retval;

	return 0;
}
