%{
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "equation.h"

#define EPS 0.00000001

extern long double equation_retval;
%}

%union{
	char *string;
	long double d;
}

%left '?' ':'
%left '>' '<' '=' NEQ
%left '-' '+'
%left '*' '/'
%left NEG
%right '^'

%token NUMBER STRING NEQ
%type <string> STRING function
%type <d> NUMBER expression
%%
full:	expression		{ equation_retval = $1; }

expression: 
			  expression '+' expression	{$$ = $1 + $3;}
			| expression '-' expression	{$$ = $1 - $3;}
			| expression '*' expression	{$$ = $1 * $3;}
			| expression '/' expression	{$$ = $1 / $3;}
			| '-' expression %prec NEG		{$$ = - $2;}
			| function '(' expression ')'	{$$ = apply_func_1 ($1, $3);}
			| function '(' expression ',' expression ')'	{$$ = apply_func_2 ($1, $3, $5);}
			| function '(' expression ',' expression ',' expression ')'	{$$ = apply_func_3 ($1, $3, $5, $7);}
			| expression '^' expression	{$$ = pow ($1, $3);}
			| '(' expression ')'				{$$ = $2;}
			| expression '?' expression ':' expression
													{$$ = ($1 + EPS < 0 || $1 - EPS > 0) ? $3 : $5;}
			| expression '>' expression	{$$ = $1 > $3;}
			| expression '<' expression	{$$ = $1 < $3;}
			| expression '=' expression	{$$ = ($1 + EPS > $3 && $1 - EPS < $3);}
			| expression NEQ expression	{$$ = ($1 + EPS < $3 || $1 - EPS > $3);}	
			| NUMBER								{$$ = $1;}

function: STRING	{$$ = $1;}
%%
