#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <ctype.h>

#include "bc.h"

#include "report.h"

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "support_usr.h"

extern GtkWidget *window1;
extern GList *outputtypes;
extern GList *parameter_list;

void
on_button_browse_clicked (GtkButton * button, gpointer user_data)
{
	openfile(GTK_WIDGET(button));
}

void openfile (GtkWidget *caller)
{
	GtkWidget *filechooserdialog=NULL;
	
	filechooserdialog = create_dialog_filechooser("Choose report");
	
	if (gtk_dialog_run (GTK_DIALOG (filechooserdialog)) ==
	    GTK_RESPONSE_ACCEPT)
	{
		char *filename;

		filename =
			gtk_file_chooser_get_filename (GTK_FILE_CHOOSER
						       (filechooserdialog));

		gtk_entry_set_text (GTK_ENTRY (lookup_widget (caller,
							      "entry_report_filename")),
									filename);

		g_free (filename);

	}

	gtk_widget_destroy (filechooserdialog);
}

void
on_window1_destroy (GtkObject * object, gpointer user_data)
{
	gtk_main_quit ();
}


void
on_open1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	openfile (GTK_WIDGET (menuitem));
}


void
on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
{

	GtkWidget *dialog_about=NULL;

	dialog_about = create_dialog_about ();

	gtk_dialog_run (GTK_DIALOG (dialog_about));
	gtk_widget_destroy (GTK_WIDGET (dialog_about));
}

void
on_button_browseoutput_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *filechooserdialog_output=NULL;

	filechooserdialog_output = create_dialog_filechooser("Choose output file");
	
	if (gtk_dialog_run (GTK_DIALOG (filechooserdialog_output)) ==
	    GTK_RESPONSE_ACCEPT)
	{
		char *filename;

		filename =
			gtk_file_chooser_get_filename (GTK_FILE_CHOOSER
						       (filechooserdialog_output));

		gtk_entry_set_text (GTK_ENTRY
				    (lookup_widget
				     (GTK_WIDGET (button),
				      "entry_outputfile")), filename);

		g_free (filename);

	}

	gtk_widget_destroy (filechooserdialog_output);

}

gboolean
on_window1_delete_event (GtkWidget * widget,
			 GdkEvent * event, gpointer user_data)
{

	if (quit_confirm ())
		gtk_main_quit ();

	return TRUE;
}

void
on_quit1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (quit_confirm ())
		gtk_main_quit ();
}

gboolean
quit_confirm ()
{
	GtkWidget *dialog_quit=NULL;
	gboolean quit;

	dialog_quit = create_dialog_quit ();

	quit = (gtk_dialog_run (GTK_DIALOG (dialog_quit)) ==
		GTK_RESPONSE_ACCEPT);

	gtk_widget_destroy (dialog_quit);

	return quit;
}

void
on_button_exec_clicked (GtkButton * button, gpointer user_data)
{
	gchar *errMsg = NULL;
	GtkWindow *dialog=NULL;
	
	errMsg = execute_ok (GTK_WIDGET (button));

	if (errMsg)
	{
		dialog = GTK_WINDOW(create_dialog_err(errMsg));
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(GTK_WIDGET(dialog));
		mem_free(errMsg);
	}
	else
		run_report ();
}


void
on_button_param_clicked (GtkButton * button, gpointer user_data)
{
	xmlDoc *doc=NULL;
	gchar *filename=NULL;
	GList *parameter_list_new = NULL;
	GtkWidget *dialog_param = NULL;
	
	filename = mem_strdup ((char *)gtk_entry_get_text (GTK_ENTRY (lookup_widget (GTK_WIDGET (button), "entry_report_filename"))));
	
	g_strstrip(filename);
	
	if (!filename[0])	/* || strlen(filename)) */
	{
		warning_dialog ("No filename entered!");
		mem_free(filename);
		return;
	}

	doc = parse_xml_doc (filename);
	mem_free(filename);
	if (!doc)
		return;

	param_getfromdoc (doc, &parameter_list_new);
	param_list_merge (&parameter_list, &parameter_list_new);
	
	free_parameter_list(&parameter_list);
	
	parameter_list = parameter_list_new;
	
	dialog_param = create_dialog_param();
	gtk_widget_show(dialog_param);
}

void
on_combobox_outputtypes_changed (GtkComboBox * combobox, gpointer user_data)
{
	GtkWidget *entry_output=NULL;
	gchar *outfile=NULL, *outfilenew=NULL, *ext = NULL;
	int active;

	entry_output =
		lookup_widget (GTK_WIDGET (combobox), "entry_outputfile");

	outfile = mem_strdup ((char *)gtk_entry_get_text (GTK_ENTRY (entry_output)));
	active = gtk_combo_box_get_active (combobox);

	if (active == -1 || !outfile[0] || isspace (outfile[0]))
		return;

	ext = g_list_nth_data (outputtypes, active);
	outfilenew = change_extension (outfile, ext);
	if (outfilenew)
	{
		gtk_entry_set_text (GTK_ENTRY (entry_output), outfilenew);
		mem_free (outfilenew);
	}
	else
		gtk_entry_set_text (GTK_ENTRY (entry_output), outfile);
	mem_free(outfile);
}

void
on_param_ok_clicked 					(GtkButton *button,
										gpointer	user_data)
{	
	GtkWidget *dialog_param=NULL;

	dialog_param = lookup_widget(GTK_WIDGET(button), "dialog_param");

	param_store_list();
	param_delete_widgets();
	
	
	gtk_widget_destroy(dialog_param);
}

void
on_param_cancel_clicked 					(GtkButton *button,
										gpointer	user_data)
{
	GtkWidget *dialog_param=NULL;
	
	param_delete_widgets();

	dialog_param = lookup_widget(GTK_WIDGET(button), "dialog_param");
	gtk_widget_destroy(dialog_param);	
}

void
on_savefav_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	GtkWidget *dialog_savefav=NULL;
	GtkWidget *entry_report_filename=NULL;
	gchar *tmp=NULL;
	
	entry_report_filename = lookup_widget(window1, "entry_report_filename");
	tmp=mem_strdup((char *)gtk_entry_get_text(GTK_ENTRY(entry_report_filename)));	
	
	dialog_savefav = create_dialog_savefav(NULL, tmp);
	
	if(gtk_dialog_run (GTK_DIALOG (dialog_savefav))==
		GTK_RESPONSE_ACCEPT)
	{
		report_savefav(-1);
	}
	gtk_widget_destroy (dialog_savefav);
	mem_free(tmp);
}

void
on_entry_report_filename_changed (GtkEditable * editable, gpointer user_data)
{
	GtkWidget *button_param=NULL;
	gchar *temp=NULL;
	GtkWidget *treeview=NULL;
	GtkTreeSelection *selection;
	
	button_param = lookup_widget (GTK_WIDGET (editable), "button_param");
	
	if (!(temp=param_ok (GTK_WIDGET (editable))))
	{
		gtk_widget_set_sensitive (button_param, TRUE); 
	}
	else
	{
		gtk_widget_set_sensitive (button_param, FALSE);
		mem_free(temp);	
	}
	treeview = lookup_widget(window1, "treeview_favourites");
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	gtk_tree_selection_unselect_all(selection);
}

void
on_entry_outputfile_changed (GtkEditable * editable, gpointer user_data)
{
	GtkWidget *button_param=NULL;
	gchar *temp=NULL;
	
	button_param = lookup_widget (GTK_WIDGET (editable), "button_param");
	
	if (!(temp=param_ok (GTK_WIDGET (editable))))
	{
		gtk_widget_set_sensitive (button_param, TRUE); 
	}
	else
	{
		gtk_widget_set_sensitive (button_param, FALSE);
		mem_free(temp);	
	}
}

void
on_treeview_favourites_row_activated   (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	GtkTreeModel *model=NULL;
	GtkTreePath *path=NULL;
	GtkTreeIter iter;
	gchar *rpath=NULL;
	GtkWidget *entry_report_filename=NULL;
	GtkTreeSelection *selection=NULL;
	
	gtk_tree_view_get_cursor(treeview, &path, NULL);
	
	model = gtk_tree_view_get_model(treeview);
	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_model_get(model, &iter, 2, &rpath, -1);
	
	entry_report_filename = lookup_widget(window1, "entry_report_filename");
	gtk_entry_set_text(GTK_ENTRY(entry_report_filename), rpath);
	
	selection = gtk_tree_view_get_selection(treeview);
	gtk_tree_selection_select_iter(selection, &iter);
}

void
on_fav_edit_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *treeview_favourites;
	GtkTreePath *p;
	GtkTreeIter iter;
	GtkTreeModel *model;
	gint id;
	gchar *name=NULL, *path=NULL;
	GtkWidget *dialog=NULL;
	
	treeview_favourites = lookup_widget(window1, "treeview_favourites");
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(treeview_favourites), &p, NULL);
	if(!p)
		return;
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview_favourites));
	gtk_tree_model_get_iter(model, &iter, p);
	gtk_tree_path_free(p);
	
	gtk_tree_model_get(model, &iter, 0, &id, 1, &name, 2, &path, -1);
	
	dialog=create_dialog_savefav (name, path);
	
	if(gtk_dialog_run (GTK_DIALOG (dialog))==
		GTK_RESPONSE_ACCEPT)
	{
		report_savefav(id);
	}
	gtk_widget_destroy (dialog);

}

void
on_fav_delete_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *treeview_favourites;
	GtkTreePath *p;
	GtkTreeIter iter;
	GtkTreeModel *model;
	
	treeview_favourites = lookup_widget(window1, "treeview_favourites");
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(treeview_favourites), &p, NULL);
	if(!p)
		return;
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview_favourites));
	gtk_tree_model_get_iter(model, &iter, p);
	gtk_tree_path_free(p);

	gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
	
	report_write_favourites(model);
	
}

void
on_entry_sf_changed						(GtkEditable	*editable,
										gpointer		user_data)
{
	GtkWidget *button_sfsave=NULL, *name=NULL, *path=NULL;
	gchar *t1=NULL, *t2=NULL;
	
	button_sfsave = lookup_widget(GTK_WIDGET(editable), "button_sfsave");
	name = lookup_widget(window1, "entry_sfname");
	path = lookup_widget(window1, "entry_sfpath");
	
	t1=mem_strdup((char *)gtk_entry_get_text(GTK_ENTRY(name)));
	t2=mem_strdup((char *)gtk_entry_get_text(GTK_ENTRY(path)));
	
	if(strlen(g_strstrip(t1)) &&
		strlen(g_strstrip(t2)))
	{
		gtk_widget_set_sensitive(button_sfsave, TRUE);
		gtk_widget_grab_default (button_sfsave);
	}
	else
		gtk_widget_set_sensitive(button_sfsave, FALSE);
	
	mem_free(t1);
	mem_free(t2);
}
