#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "bc.h"

#include "init.h"

#include "interface.h"
#include "support.h"
#include "support_usr.h"

extern gchar *config_path;
extern gchar *fav_file;
extern GtkWidget *window1;
extern GList *outputtypes;

/* return TRUE on success */
gboolean
init_populate_favourites(gchar *favourites_file, GtkWidget **treeview)
{
	FILE *file=NULL;
	int row=0;
	gchar buf[256];
	GtkListStore *store_favourites=NULL;
	GtkTreeIter iter_fav;
	GtkTreeViewColumn *tree_view_column_name=NULL;
	GtkTreeViewColumn *tree_view_column_path=NULL;
	GtkCellRenderer *renderer_name=NULL, *renderer_path=NULL;
	
	gchar *name=NULL, *path=NULL;
	
	g_assert(favourites_file);
	g_assert(treeview);
	
	if(!(file = fopen(favourites_file, "a+")))
	{
		warning_dialog("Could not open/create favourites file: %s\nFavourites will not work\n");
		fprintf(stderr, "Could not open/create favourites file: %s\nFavourites will not work\n", favourites_file);
		return FALSE;
	}	
	
	store_favourites = gtk_list_store_new (3, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING);
	gtk_tree_view_set_model(GTK_TREE_VIEW(*treeview), GTK_TREE_MODEL(store_favourites));

	while(fgets(buf, 255, file))
	{
		if(buf[0]=='#' || isspace(buf[0]))
			continue;
	
		name=mem_strdup(buf);
		
		/* get next line (path) */
		if(!fgets(buf, 255, file))
			break;

		path=mem_strdup(buf);

		/* trim newlines from end */
		name[strlen(name)-1]=0;
		path[strlen(path)-1]=0;

		/* add row and fill columns */
		gtk_list_store_append(store_favourites, &iter_fav);
		gtk_list_store_set (store_favourites, &iter_fav,	
														0, row, 
														1, name,
														2, path,
														-1);
		
		row++;
		mem_free(name);
		mem_free(path);
	}
	fclose(file);

	//renderer_hidden = gtk_cell_renderer_text_new ();
//	tree_view_column_hidden = gtk_tree_view_column_new ();
//	gtk_tree_view_column_set_title(tree_view_column_hidden, "ID");
	//tree_view_column_hidden = gtk_tree_view_column_new_with_attributes ("ID", renderer_hidden, "text", 0, NULL);
	
	renderer_name = gtk_cell_renderer_text_new();
	tree_view_column_name = gtk_tree_view_column_new_with_attributes ("Name", renderer_name, "text", 1, NULL);
	g_object_set (G_OBJECT (renderer_name), "weight", "bold", NULL);
	
	renderer_path = gtk_cell_renderer_text_new();
	tree_view_column_path = gtk_tree_view_column_new_with_attributes ("Path", renderer_path, "text", 2, NULL);
	
//	gtk_tree_view_append_column (GTK_TREE_VIEW(*treeview), tree_view_column_hidden);
	gtk_tree_view_append_column (GTK_TREE_VIEW(*treeview), tree_view_column_name);
	gtk_tree_view_append_column (GTK_TREE_VIEW(*treeview), tree_view_column_path);
	
	return TRUE;
}

gboolean init_setup(int argc, char *argv[])
{
	GtkWidget *treeview_favourites=NULL;
	GtkWidget *combobox_outputtype=NULL;
	GList *walk=NULL;

	int tmp=0;
	gchar *home=NULL, *pos=NULL, *temp=NULL;
	
	outputtypes = g_list_append (outputtypes, "PDF");
	outputtypes = g_list_append (outputtypes, "PS");
	outputtypes = g_list_append (outputtypes, "TEX");
	outputtypes = g_list_append (outputtypes, "HTML");
	outputtypes = g_list_append (outputtypes, "TXT");
	outputtypes = g_list_append (outputtypes, "XML");

	combobox_outputtype = lookup_widget (window1, "combobox_outputtypes");

	for (walk = g_list_first (outputtypes); walk != NULL;
	     walk = walk->next)
	{
		gtk_combo_box_append_text (GTK_COMBO_BOX (combobox_outputtype),
					   _((gchar *) walk->data));
	}

	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_outputtype), 0);

	home = getenv("HOME");
	
	config_path = mem_strdup(bcconfig_getvalue("papyrus_config_path"));
	if(!config_path)
		config_path=mem_strdup("~/.papyrus");
	
	/* replace '~' (=126) with $HOME */
	pos=strchr(config_path, 126);
	if(pos)
	{
		temp = mem_strdup_printf("%s%s", home, &(pos[1]));
		mem_free(config_path);
		config_path=temp;
	}
	
	if(argv[1])
		fav_file = mem_strdup(argv[1]);
	else
	{
		temp = mem_strdup(bcconfig_getvalue("favourites_file"));
		if(!temp)
			temp = mem_strdup("example.fav");
		fav_file = mem_strdup_printf("%s/%s", config_path, temp);
		mem_free(temp);
	}
  	/* replace '~' (=126) with $HOME */
	pos=strchr(fav_file, 126);
	if(pos)
	{
		temp = mem_strdup_printf("%s%s", home, &(pos[1]));
		mem_free(fav_file);
		fav_file=temp;
	}
	if(!g_file_test(config_path, G_FILE_TEST_EXISTS))
	{
		tmp=mkdir(config_path, 0755);
		printf("creating directory %s returned %d\n", config_path, tmp);
	}
 
	treeview_favourites = lookup_widget(window1, "treeview_favourites");
	
	init_populate_favourites(fav_file, &treeview_favourites);
	
	
	return TRUE;
}
