#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _XOPEN_SOURCE /* for strptime, glibc2 needs this */

#include <libxml/xmlreader.h>
#include <gtk/gtk.h>
#include <time.h>
#include <ctype.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "bc.h"

#include "report.h"

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "support_usr.h"
#include "binding.h"

extern gchar *fav_file;
extern gchar *config_path;
extern GList *parameter_list;
extern GList *databases;
extern GtkWidget *window1;
extern GList *outputtypes;

gboolean run_report()
{
	gchar *command=NULL, *ext=NULL, *outfile=NULL, *infile=NULL, *tmp=NULL;
	GtkWidget *entry_output=NULL, *entry_report_filename=NULL;
	GtkWidget *combobox=NULL, *checkbutton_openviewer=NULL, *checkbutton_validate=NULL;
	GList *walk=NULL;
	require_parameter *p=NULL;	
	int outputtype_active=-1;
//	GtkWindow *dialog;
	
	command = mem_strdup(bcconfig_getvalue("report_binary"));
	
	if(parameter_list)
	{
		walk = g_list_first(parameter_list);
		while(walk)
		{
			p = (require_parameter*)walk->data;
			
 			if(p->value)
			{
				tmp = mem_strdup_printf("%s -a %s=%s", command, 
					p->ident, p->value);
				mem_free(command);
				command=tmp;
			}
			walk = walk->next;
		}
	}
	else
	{
		tmp = mem_strdup(command);
		mem_free(command);
		command=tmp;
	}
	
	entry_output = lookup_widget (GTK_WIDGET (window1), "entry_outputfile");
	entry_report_filename = lookup_widget (GTK_WIDGET (window1), "entry_report_filename");
	combobox = lookup_widget (GTK_WIDGET(window1), "combobox_outputtypes");
	checkbutton_openviewer = lookup_widget (GTK_WIDGET(window1), "checkbutton_openviewer");
	checkbutton_validate = lookup_widget (GTK_WIDGET(window1), "checkbutton_validate");
	
	outfile = mem_strdup ((char *)gtk_entry_get_text (GTK_ENTRY (entry_output)));
	outfile = g_strstrip(outfile);
	infile = mem_strdup ((char *)gtk_entry_get_text (GTK_ENTRY (entry_report_filename)));
	infile = g_strstrip(infile);
	outputtype_active = gtk_combo_box_get_active (GTK_COMBO_BOX(combobox));

	if (outputtype_active == -1 || !outfile[0] || isspace (outfile[0]))
		ext=mem_strdup("ps");
	else
		ext=mem_strdup(g_list_nth_data (outputtypes, outputtype_active));
	
	tmp = mem_strdup_printf("%s -t %s -o %s", command, g_ascii_strdown(ext, -1), outfile);
	mem_free(ext);
	mem_free(outfile);
	mem_free(command);
	command=tmp;

	tmp = mem_strdup_printf("%s %s", command, infile);
	mem_free(command);
	mem_free(infile);
	command=tmp;
	
	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbutton_validate))
		==TRUE)
	{
		tmp = mem_strdup_printf("%s -d", command);
		mem_free(command);
		command=tmp;
	}
	
	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbutton_openviewer))
		==TRUE)
	{
		tmp = mem_strdup_printf("%s -v", command); 
		mem_free(command);
		command=tmp;
	}
/*	
	dialog = GTK_WINDOW(create_dialog_err(command));
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(GTK_WIDGET(dialog));
*/	
	printf("Running report\n%s\n", command);
	system(command);
	printf("Finished\n");
	
	mem_free(command);

	return TRUE;
}

/**
	execute_ok
	returns reason why report can't be executed or NULL if it can
*/
gchar *
execute_ok(GtkWidget *start)
{
	gchar *retval = NULL;
	gchar *tmp=NULL, *tmp2=NULL;
	GList *walk=NULL;
	require_parameter *p=NULL;

	g_assert(start);
	
	g_assert(start);
	
	retval = mem_strdup("");

	/* check if parameters are filled in */
	if(!parameter_list || !g_list_length(parameter_list))
	{
		tmp = mem_strdup_printf("%sEnter parameters before running report\n", retval);
		mem_free(retval);
		retval=tmp;
	}
	else
	{
		walk = g_list_first(parameter_list);
		while(walk)
		{
			p=(require_parameter*)walk->data;
			if(!p->value || !strlen(p->value))
			{
				tmp = mem_strdup_printf("%sEnter all parameters before running report\n", retval);
				mem_free(retval);
				retval=tmp;
				break;
			}
			walk=walk->next;
		}
	}
	
	if((tmp = param_ok(start)))
	{
		tmp2 = retval;
		retval = mem_strdup_printf("%s%s", tmp2, tmp);
		mem_free(tmp2);
	}
	mem_free(tmp);
	
	if(strlen(retval))
		return retval;
	else 
	{
		mem_free(retval);
		return NULL;
	}
}
/* checking to see if there is sufficient *
 * information to open parameter window   */
gchar *
param_ok(GtkWidget *start)
{
	gchar *retval = NULL;
	gchar *tmp = NULL;
	GtkWidget *entry_report_filename = NULL;
	GtkWidget *entry_outputfile = NULL;

	g_assert(start);
		
	/* check input and output fields have values */
	entry_report_filename = lookup_widget(start, "entry_report_filename");
	entry_outputfile = lookup_widget(start, "entry_outputfile");

	retval = mem_strdup("");

	tmp = mem_strdup((char *)gtk_entry_get_text(GTK_ENTRY(entry_report_filename)));
	g_strstrip(tmp);
	if(!strlen(tmp))
	{
		mem_free(tmp);
		tmp = mem_strdup_printf("%sEnter report filename\n", retval);
		mem_free(retval);
		retval=tmp;
	}
	else
	mem_free(tmp);

	tmp = mem_strdup((char *)gtk_entry_get_text(GTK_ENTRY(entry_outputfile)));
	g_strstrip(tmp);
	if(!strlen(tmp))
	{
		mem_free(tmp);
		tmp = mem_strdup_printf("%sEnter output filename\n", retval);
		mem_free(retval);
		retval=tmp;
	}
	else
		mem_free(tmp);
	
	if(strlen(retval))
		return retval;
	else
	{
		mem_free(retval);
		return NULL;	
	}
}

xmlDoc *
parse_xml_doc(gchar * xmlfilename)
{
	xmlDoc *doc = NULL;
	gchar *err= NULL;
	xmlTextReaderPtr reader = NULL;
	int ret;

	g_assert(xmlfilename);
	
	xmlInitParser();
	reader = xmlReaderForFile(xmlfilename, NULL, 0);
	if (reader != NULL) 
	{
		ret = xmlTextReaderRead(reader);
		while (ret == 1) 
		{
			//processNode(reader);
			ret = xmlTextReaderRead(reader);
		}
		xmlFreeTextReader(reader);
		if (ret != 0)
		{
			err = mem_strdup_printf("%s : Failed to parse\n", xmlfilename);
		}
	} 
	else 
	{
		err = mem_strdup_printf("%s : Unable to open\n", xmlfilename);
	}

	if(err)
	{
		fprintf(stderr, err);
	  	warning_dialog(err);
		mem_free(err);
		return NULL;
	}
	doc = xmlParseFile(xmlfilename);

	return doc;
}

/**
  * param_getfromdoc: gets the required fields for the report
  * <requires>blah</requires>
  * populates param_list GList of parameters
  */
void
param_getfromdoc(xmlDoc *doc, GList **param_list)
{
	xmlNode *node=NULL;
  
	g_assert(doc);
	g_assert(param_list);
	
	/* just in case binding_init has already been called,
		dirty but it works */
	binding_cleanup();
	
	if(binding_init(doc))
		fprintf(stderr, "Couldn't initialise databse binding\n");
	
	node = xmlDocGetRootElement(doc);
  
	free_databases();
	param_addtolist(node, param_list);
}

void
param_addtolist(xmlNode * a_node, GList **list)
{
	require_parameter *param = NULL;
	xmlNode *cur_node = NULL;
	gchar *tmp = NULL;
	Database_dd *db=NULL;
	g_assert(a_node);
	g_assert(list);
	
	for (cur_node = a_node; cur_node; cur_node = cur_node->next) 
	{
		if ((cur_node->type == XML_ELEMENT_NODE) && 
		(!strcmp("requires", cur_node->name)))
		{
			param = mem_calloc(sizeof(require_parameter));
	
			tmp = xmlNodeGetContent(cur_node);
			if(tmp)
				param->ident = mem_strdup(tmp);
			else 
				continue;
			tmp = xmlGetProp(cur_node, "name");
			if(tmp)
				param->label = mem_strdup(tmp);
			else
				param->label = mem_strdup(param->ident);
		
			tmp = xmlGetProp(cur_node, "type");
			if(tmp)
				param->type = param_type_from_str(tmp);

			if(param->type == TYPE_COMBO)
			{
				tmp = xmlGetProp(cur_node, "database");
				if(tmp)
				{
					param->database = mem_alloc(sizeof (Database_dd));
					param->database->name=mem_strdup(tmp);
				}
				tmp = xmlGetProp(cur_node, "sql");
				if(tmp)
					param->sql = mem_strdup(tmp);
			}
		
			param->value=NULL;
			*list = g_list_append(*list, param);
		}
		else if ((cur_node->type == XML_ELEMENT_NODE) && 
			(!strcmp("database", cur_node->name)))
		{
			db = mem_alloc(sizeof(Database_dd));
			tmp = xmlNodeGetContent(cur_node);
			if(tmp)
				db->c_string = mem_strdup(tmp);
			else
			{
				mem_free(db);
				printf("Couldn't get database:sql, ignoring <database> entry\n");
				continue;
			}
			tmp=xmlGetProp(cur_node, "type");
			if(tmp)
				db->type=mem_strdup(tmp);
			else
			{
				mem_free(db);
				printf("Couldn't get database:type, ignoring <database> entry\n");
				continue;
			}
			tmp = xmlGetProp(cur_node, "name");
			if(tmp)
				db->name=mem_strdup(tmp);
			else
			{
				mem_free(db);
				printf("Couldn't get database:name, ignoring <database> entry\n");
				continue;
			}
			
			databases = g_list_append(databases, db);
			printf("Got database:%s, c_string: %s, type: %s\n", db->name, g_strstrip(db->c_string), 
					db->type);
		}
	
	if(cur_node->children)
		param_addtolist(cur_node->children, list);
	}
}

void free_database(Database_dd *db)
{
	if(!db)
		return;
	mem_verify(db);
	
	if(db->name)
	{
		mem_free(db->name);
		db->name=NULL;
	}
	if(db->type)
	{
		mem_free(db->type);
		db->type=NULL;
	}
	if(db->c_string)
	{
		mem_free(db->c_string);
		db->c_string=NULL;
	}
	mem_free(db);
	db=NULL;
}

void
free_databases()
{
	GList *walk=NULL;
	Database_dd *db=NULL;
	
	if(!databases)
		return;
	
	walk = g_list_first(databases);
	while(walk)
	{
		db = (Database_dd*)walk->data;
		free_database(db);
		walk=walk->next;
	}
	
	g_list_free(databases);
	databases=NULL;
}

void
free_parameter_list(GList **list)
{
	GList *w=NULL;
	require_parameter *p=NULL;
	
	if(!*list)
		return;
	
	w = g_list_first(*list);
	for (;w!=NULL;w=w->next)
	{
		p = w->data;
		free_require_parameter(p);
	}
	g_list_free(*list);
	*list=NULL;
}

void
free_require_parameter(require_parameter *p)
{
	GList *w=NULL;
	row_dd *r=NULL;
	if(!p)
	{
		message("not freeing parameter because it's NULL");
		return;
	}
	mem_free(p->ident);
	p->ident=NULL;
	
   	mem_free(p->label);
	p->label=NULL;
   	
	if (p->value)
	mem_free(p->value);
	p->value=NULL;
	mem_free(p->sql);
	p->sql=NULL;
	
	/* don't need to free p->database because it points to a member 
		of the global GList *databases which is freed separately */
	p->database=NULL;

	w = g_list_first(p->datalist);
   	for (;w!=NULL;w=w->next)
		{
		r = w->data;
		mem_free(r->name);
		mem_free(r);
		}
	g_list_free(p->datalist);
	p->datalist=NULL;
	
	mem_free(p);
	}

gchar *
change_extension(gchar *oldfilename, gchar *ext)
{
  gchar *newfilename = NULL;
  int slen, i;

  g_assert(oldfilename);
	
  if (!ext)
	  return NULL;
  slen = strlen(oldfilename);
  for (i=slen-1;i>0;i--)
	  if (oldfilename[i]=='.')
		{
		oldfilename[i] = 0;
		break;
	  	}
  
  
  newfilename = mem_strdup_printf("%s.%s",oldfilename,g_ascii_strdown(ext, -1));
  
  /*printf("newfilename: %s, ext: %s len: %d\n", 
		 newfilename, ext, strlen(newfilename));*/
  
  return newfilename;
}

/* requires 2 x Glist of requires_param
 * if param exists in both, copy value to new list
 */
void
param_list_merge(GList **list_cur, GList **list_new)
{
	GList *walk_cur = NULL, *walk_new = NULL;
	require_parameter *p_cur = NULL, *p_new = NULL;

	g_assert(list_cur);
	g_assert(list_new);	

	/* 	iterate thru list_new checking if we want to get value
	 *	from list_cur
	*/
	for(walk_new=g_list_first(*list_new); walk_new; walk_new=walk_new->next)
	{
		p_new = walk_new->data;
		for(walk_cur = g_list_first(*list_cur); walk_cur; walk_cur=walk_cur->next)
		{
			p_cur = walk_cur->data;
			if(p_new->ident && p_cur->ident && !strcmp(p_new->ident, p_cur->ident))
			{
			   
			   if (!p_cur->value)
			     continue;
			   p_new->value = mem_strdup(p_cur->value);
			   
			   break;
			}
		}
	}
}

int param_type_from_str(gchar *str)
{
	if(!str)
		return TYPE_TEXT;
	if(!strcmp(str, "int"))
		return TYPE_INT;
	if(!strcmp(str, "date"))
		return TYPE_DATE;
	if(!strcmp(str, "dropdown") ||
		!strcmp(str, "combo"))
		return TYPE_COMBO;

	return TYPE_TEXT;
}

int param_store_list()
{
	GList *w;
	require_parameter *p = NULL;
	guint year=0, month=0, day=0, active=-1;
	row_dd *row;
	
	for(w=g_list_first(parameter_list); w; w=w->next)
	{
		p = w->data;
	   	if (p->value)
	     		{
			mem_free(p->value);
			p->value = NULL;
			}
	   
		
		switch(p->type)
		{
			case TYPE_DATE:
				gtk_calendar_get_date(GTK_CALENDAR(p->widget), &year, &month, &day);
				month++; /* (JAN = 0, DEC = 11, stupid computers) */
				if(year && month && day)
					p->value = mem_strdup_printf("%d-%02d-%02d", year, month, day);
				break;
			case TYPE_INT:
				p->value = mem_strdup_printf("%d", 
					gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(p->widget)));
				break;
			case TYPE_COMBO:
				active=gtk_combo_box_get_active(GTK_COMBO_BOX(p->widget));
				if(active != -1)
				{
					row = g_list_nth_data(p->datalist, active);
					if(row)
						p->value = mem_strdup_printf("%d", row->id);
				}
				break;
			case TYPE_TEXT:
			default:
				p->value = mem_strdup((char *)gtk_entry_get_text(GTK_ENTRY(p->widget)));
				if(p->value)
					g_strstrip(p->value);
		}
	}

	return 0;
}

/* if row >=0, edit row, else create new */
void report_savefav(gint row)
{
	GtkWidget *treeview_favourites=NULL, *entry_sfname=NULL, *entry_sfpath=NULL;
	GtkTreeIter iter;
	GtkTreeModel *model=NULL; 
	gchar *name=NULL, *path=NULL;
	int temp=0;
	int d=1;
	
	treeview_favourites = lookup_widget(window1, "treeview_favourites");
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview_favourites));

	entry_sfname = lookup_widget(window1, "entry_sfname");
	entry_sfpath = lookup_widget(window1, "entry_sfpath");
	name = mem_strdup((char *)gtk_entry_get_text(GTK_ENTRY(entry_sfname)));
	path = mem_strdup((char *)gtk_entry_get_text(GTK_ENTRY(entry_sfpath)));
	
	if(row < 0)
	{
		/* new row */
		/* find new id */
		d=gtk_tree_model_get_iter_first(model, &iter);
		while(d)
		{
			gtk_tree_model_get(model, &iter, 0, &temp, -1);
			d=gtk_tree_model_iter_next(model, &iter);
		}
		temp = temp+1;
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		gtk_list_store_set (GTK_LIST_STORE(model), &iter,	
													0, temp, 
													1, name,
													2, path,
													-1);
	}
	else
	{
		/* edit row */
		gtk_tree_model_get_iter_first(model, &iter);
		gtk_tree_model_get(model, &iter, 0, &temp, -1);
		while(temp != row)
		{
			//message("temp:%d, row:%d\n", temp, row);
			gtk_tree_model_iter_next(model, &iter);
			gtk_tree_model_get(model, &iter, 0, &temp, -1);
		}
			gtk_list_store_set (GTK_LIST_STORE(model), &iter,	
														1, name,
														2, path,
														-1);
	}	
	report_write_favourites(model);
	mem_free(name);
	mem_free(path);
}	


void report_write_favourites(GtkTreeModel *model)
{
	GtkTreeIter iter;
	FILE *f=NULL;
	gchar *name=NULL, *path=NULL;
	int t=0;
	/* write new treeview data to file */
	if(!(f = fopen(fav_file, "w+")))
	{
		mkdir(config_path, 0755);
		if(!(f = fopen(fav_file, "w+")))
		{
		warning_dialog("Could not open file for writing, favourites not saved.");
		fprintf(stderr, "Could not open file for writing: %s\n", fav_file);
		return;
		}
	}
	
	fprintf(f, "# name\n#/path/to/report.xml\n\n");

	t = gtk_tree_model_get_iter_first(model, &iter);
	while(t)
	{
		gtk_tree_model_get(model, &iter, 
												1, &name, 
												2, &path, 
												-1);
		//printf("%s\n%s\n\n", name, path);
		fprintf(f, "%s\n%s\n\n", name, path);
		t= gtk_tree_model_iter_next(model, &iter);
	}
	
	if(name)
		g_free(name);
	if(path)
		g_free(path);
	
	fclose(f);
}

int date_from_str(char *str, int *year, int *month, int *day)
{
		time_t t;
		struct tm tm;
		char buf[255];
		
		g_assert(str);
		
		t = time(0);
		memset(&tm,0,sizeof(struct tm));
		memcpy(&tm,localtime(&t),sizeof(struct tm));
		if(!strptime(str, "%Y-%m-%d", &tm))
			return -1;

		strftime(buf, sizeof(buf), "%d", &tm);
		*day = atoi(buf);
		strftime(buf, sizeof(buf), "%m", &tm);
		*month = atoi(buf);
		strftime(buf, sizeof(buf), "%Y", &tm);
		*year = atoi(buf);
		
		return 0;
}

void
print_param_list(GList *list)
{
	GList *w=NULL;
	require_parameter* p=NULL;
	
	for(w=g_list_first(list);w;w=w->next)
	{
		p=w->data;
	   	mem_verify(p);
		printf("%s:\tt=%d\tv=%p %s\n",
			p->ident, p->type, p->value, p->value);
	   if (p->value)
	     mem_verify(p->value);
	}
}
