#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <libxml/parser.h>
#include <libxml/xmlmemory.h>

#include "html_output.h"
#include "xml_funcs.h"
#include "debug.h"

int output_cell_start (FILE *output)
{
	return fprintf (output, "<TD>");
}

int output_cell_end (FILE *output)
{
	return fprintf (output, "</TD>");
}

int output_row_start (FILE *output)
{
	return fprintf (output, "<TR>");
}

int output_row_end (FILE *output)
{
	return fprintf (output, "</TR>\n");
}

int output_pageheader_start (FILE *output)
{
	return fprintf (output, "<B>");
}

int output_pageheader_end (FILE *output)
{
	return fprintf (output, "</B>");
}

int output_header (FILE *output, char *text)
{
	return fprintf (output,
	"<HTML>\n"
	"<HEAD>\n"
	"	<TITLE>%s</TITLE>\n"
	"	<STYLE TYPE=\"text/css\">\n"
	"		BODY  { font-size: 11pt }\n"
	"		BIG   { font-size: 18pt }\n"
	"		SMALL { font-size: 9pt }\n"
	"	</STYLE>\n"
	"</HEAD>\n\n"
	"<BODY>\n", text);
}

int output_footer (FILE *output, char *text)
{
	int retval = 0;

	if (text)
		retval = fprintf (output, "<H6>%s</H6>\n", text);
	retval += fprintf (output, "</BODY>\n</HTML>\n");

	return retval;
}


char *html_convert_width (char *width)
{
	char *retval;
	
	/* Francis: I guess valid form are screen portion "100%" or pixel "100" */
	/* I can't think of any good idea to get full browser screen pixel width */
	/* I assume 800x600 environment */
	if (strchr (width, '%') != 0)
		asprintf (&retval, "%d", 800 * atoi (width) / 100);
	else
		asprintf (&retval, "%s", width);
	
	return retval;
}

char *html_convert_height (char *height)
{
	char *retval;
	
	/* Francis: I guess valid form are screen portion "100%" or pixel "100" */
	/* I can't think of any good idea to get full browser screen pixel height */
	/* I assume 800x600 environment */
	if (strchr (height, '%') != 0)
		asprintf (&retval, "%d", 600 * atoi (height) / 100);
	else
		asprintf (&retval, "%s", height);
	
	return retval;
}

int html_string(FILE *output, char *text)
{
/*
	int i;
	fputs("\n\n", stderr);
	for (i = 0; *(text+i); i++) {
		fprintf(stderr, "%d(%c) ", *(text+i), *(text+i));
	}
*/
	for (; *text; text++) {
		/* Trim trailing spaces */
		int i;
		/* Forward if space character keep occurring */
		for (i = 0; *(text+i); i++) {
			if (!isblank(*(text+i)) ) { break; }
		}
		if (!*(text+i)) {
			return 0;
		}

		/* For DOS coding */
		if (*text == '\r' && *(text+1) == '\n') {
			fputs("<BR>", output);
			text++;
		} else if (*text == '\n') {
			fputs("<BR>", output);
		} else if (*text == '>') {
			fputs("&gt;", output);
		} else if (*text == '<') {
			fputs("&lt;", output);
		} else if (isspace(*text)) {
			/* If next is also blank, keep forwarding */
			while ( isspace(*(text+1)) ) {
				text++;
			}
			fputc(' ', output);
		} else {
			fputc(*text, output);
		}

	}

	return 0;
}


int html_image (FILE *output, xmlNode *image)
{
	char *imagefile;
	char *width, *height;
	char *tempstr;

	width = xml_node_parent_attr (image, "width");
	height = xml_node_parent_attr (image, "height");

	imagefile = xml_node_attr_val (image, "file");
	fprintf (output, "<IMG SRC=\"%s\"", imagefile);
	
	if (width)
		{
		tempstr = html_convert_width (width);
		fprintf (output, " width=\"%s\"", tempstr);
		free (tempstr);
		}

	if (height)
		{
		tempstr = html_convert_height (height);
		fprintf (output, " height=\"%s\"", tempstr);
		free (tempstr);
		}

	fputs (">", output);
	

	return 0;
}

static FILE *myoutput;

void html_set_output(FILE *output)
{
	myoutput = output;
}

int wrap(int length,
	int *conds, /* array of boolean (!0 => true, 0 => false) conditions */
	char **begintags,
	char **endtags,
	int depth_start,
	int (*f)()
)
{
	int i, j;
	char indent[] = "  ";

	
	for (i = 0; i < length; i++) {
		if (conds[i]) {
			for (j = 0; j < depth_start; j++) { fputs(indent, myoutput); }
			fputs(begintags[i], myoutput);
			fputc('\n', myoutput);
			depth_start++;
		}
	}

	f();
	fputc('\n', myoutput);

	for (i = length - 1; i >=0; i--) { 
		if (conds[i]) {
			for (j = 0; j < depth_start; j++) { fputs(indent, myoutput); }
			fputs(endtags[i], myoutput);
			fputc('\n', myoutput);
			depth_start--;
		}
	}

	return 0;
}


/*
 * Francis: This function provides a systematic and managable way
 *          output complicated surrounding tag printing coding.
 *
 * My previous code was like this:
 * if (isbold) {
 *   fputs("<b>");
 *   if (isitalic) {
 *     fputs("<i>");
 *     if (font) {
 *       fputs("<font face=Times>");
 *       <print text content>
 *       fputs<"</font>");     
 *     }
 *     fputs("</i>");
 *     else {
 *       <print text content>
 *     }
 *   }
 *   else {
 *     if (font) {
 *       fputs("<font face=Times>");
 *       <print text content>
 *       fputs<"</font>");     
 *     }
 *     else {
 *       <print text content>
 *     }
 *   }
 *   fputs("</b>");
 * }
 * else {
 *   ...
 *
 *  As you can see, it gets rapidly a big mess....
 */
int html_label(FILE *output, xmlNode *label)
{
    xmlNode *textnode;
    xmlNode *item;
    int isbold, isitalic, isparagraph;
    char *text, *font, *align, *width , *size;
	int i, result;

#define DEPTH 6
	int conds[DEPTH];
	char *begintags[DEPTH];
	char *endtags[DEPTH];

	static int handle_content();

	/* Nested function */
	static int handle_content () {
		/* Handle non-recursive atomic nodes */
		for (item = label->children; item; item = item->next) {
			if (!strcmp (item->name, "text")) {
				html_string (output, item->content);
			} else if (!strcmp (item->name, "image")) {
				html_image (output, item);
			} else if (!strcmp (item->name, "comment")) {
				;
			} else {
				errormsg ("Invalid label item: %s", item->name);
			}
		}

		return 0;
	}

    textnode = label->children;
    text = "";
    if (textnode)
        if (textnode->content)
            text = textnode->content;
   
	/* If they have boolean type */
    isbold      = xml_node_parent_attr_val (label, "bold", "true");
    isitalic    = xml_node_parent_attr_val (label, "italic", "true");
	isparagraph = xml_node_parent_attr_val (label, "paragraph", "true");

	font        = xml_node_parent_attr (label, "font");
	align       = xml_node_parent_attr (label, "align");
	width       = xml_node_parent_attr (label, "width");
	size        = xml_node_parent_attr (label, "size");


	conds[0] = isparagraph;
	conds[1] = (int)(align);
	conds[2] = isbold;
	conds[3] = isitalic;
	conds[4] = (int)(size);
	conds[5] = (int)(font);

	
	/* begin tags */

	asprintf(&begintags[0], "\n<!-- begintag --><p>");
	if (align) {
		asprintf(&begintags[1], "<span align=\"%s\">", align);
	} else {
		begintags[1] = NULL;
	}
	asprintf(&begintags[2], "<b>");
	asprintf(&begintags[3], "<i>");
	/* size = { big, small } */
	if (size) {
		if (!strcmp(size, "big") || !strcmp(size, "small")) {
			asprintf(&begintags[4],"<%s>", size);
		}
	}
	else {
		begintags[4] = NULL;
	}

	/* font = { Helvetica, Lucida, etc } */
	if (font) {
		asprintf(&begintags[5],"<font face=\"%s\">", font);
	} else {
		begintags[5] = NULL;
	}

	/* end tags */

	asprintf(&endtags[5], "</font>");
	if (size) {
		if (!strcmp(size, "big") || !strcmp(size, "small")) {
			asprintf(&endtags[4],"</%s>", size);
		}
	}
	else {
		endtags[4] = NULL;
	}
	asprintf(&endtags[3], "</i>");
	asprintf(&endtags[2], "</b>");
	asprintf(&endtags[1], "</span>");
	/*asprintf(&endtags[0], "</p>");*/
	asprintf(&endtags[0], "<!-- place of </p> -->");

	html_set_output(output);

	result = wrap(DEPTH, conds, begintags, endtags, 1, handle_content);
	
	/* free */
	for (i = 0; i < DEPTH; i++) {
		if (begintags[i]) {
			free(begintags[i]);
		}
		if (endtags[i]) {
			free(endtags[i]);
		}
	}

	return result;
}

