<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">
  <xsl:output method="html" indent="no"
    doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"
    doctype-public="-//W3C//DTD XHTML 1.0 Transitional//EN" />

  <xsl:template match="/">
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match="report">
    <html>
      <xsl:if test="description/text()!=''">
        <xsl:comment>
          <xsl:value-of select="description" />
        </xsl:comment>
      </xsl:if>

      <head>
        <style type="text/css">
          body  { font-size: 12pt }
          big   { font-size: 18pt }
          small { font-size: 9pt }
   	  table { empty-cells: show;
	  border-collapse:collapse; }
	  th {background-color: #BBBBFF; }
        </style>
      </head>
      <body>
        <xsl:apply-templates />
      </body>
    </html>
  </xsl:template>

  <xsl:template match="title">
    <title>
      <xsl:choose>
        <xsl:when test=".!=''">
          <xsl:apply-templates />
        </xsl:when>
        <xsl:when test="../@name!=''">
          <xsl:value-of select="../@name" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>Papryus Report</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </title>
  </xsl:template>

  <xsl:template match="table">
    <xsl:element name="table">
      <xsl:copy-of select="@border" />
      <xsl:copy-of select="@width" />
      <xsl:copy-of select="@rules" />
      <xsl:copy-of select="@frame" />
      
      <xsl:apply-templates select="header" />
      <xsl:apply-templates select="footer" />
      <xsl:call-template name="body" />
    </xsl:element>
  </xsl:template>

  <xsl:template match="header">
    <thead><xsl:apply-templates/></thead>
  </xsl:template>

  <xsl:template match="footer">
    <tfoot><xsl:apply-templates/></tfoot>
  </xsl:template>

  <xsl:template name="body">
    <tbody><xsl:apply-templates select="row" /></tbody>
  </xsl:template>

  <xsl:template match="row">
    <tr><xsl:apply-templates/></tr>
  </xsl:template>

  <xsl:template match="cell">
    <xsl:variable name="cell-tag">
      <xsl:choose>
        <xsl:when test="name(../..)='header'">th</xsl:when>
        <xsl:when test="name(../..)='footer'">td</xsl:when>
        <xsl:when test="name(../..)='table'">td</xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="$cell-tag!=''">
      <xsl:element name="{$cell-tag}">
        <xsl:if test="@span!=''">
          <xsl:attribute name="colspan">
            <xsl:value-of select="@span" />
          </xsl:attribute>
        </xsl:if>
        <xsl:copy-of select="@align" />
        <xsl:copy-of select="@width" />
        <xsl:apply-templates />
      </xsl:element>
    </xsl:if>
  </xsl:template>

  <xsl:template match="line">
    <xsl:variable name="width">
      <xsl:choose>
        <xsl:when test="@width!=''">
          <xsl:value-of select="@width" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'100%'" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="style">
      <xsl:choose>
        <xsl:when test="@style!=''">
          <xsl:value-of select="@style" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'single'" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <hr width="{$width}" />
    <xsl:if test="$style='double'">
      <hr width="{$width}" />
    </xsl:if>
    <br />
  </xsl:template>

  <xsl:template match="spacer">
    <xsl:variable name="style">
      <xsl:choose>
        <xsl:when test="@style!=''">
          <xsl:value-of select="@style" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'vertical'" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!-- Where xml2html.c converts '123cm' to 3 <br/> elements, this does
         no such thing, assuming that the number specified is correct. -->
    <!--    <xsl:variable name="size" select="number(substring-before(.,'cm'))" /> -->
    <xsl:call-template name="spaces">
      <xsl:with-param name="size" select="number(substring-before(.,'cm'))" />
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="spaces">
    <xsl:param name="size" />
    <xsl:choose>
      <xsl:when test="$size > 0">
        <br />
        <xsl:call-template name="spaces">
          <xsl:with-param name="size" select="$size - 1" />
        </xsl:call-template>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="image">
    <xsl:variable name="filename">
      <xsl:variable name="fileref">
        <xsl:choose>
          <xsl:when test="@file!=''">
            <xsl:value-of select="@file" />
          </xsl:when>
          <xsl:when test=".!=''">
            <xsl:value-of select="." />
          </xsl:when>
        </xsl:choose>
      </xsl:variable>
      <xsl:if test="$fileref">
        <xsl:choose>
          <xsl:when test="contains($fileref,'.')">
            <xsl:value-of select="$fileref" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="concat($fileref, '.png')" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
    </xsl:variable>
    <xsl:if test="$filename">
      <img src="{$filename}">
        <xsl:if test="@width!=''">
          <xsl:variable name="width">
            <xsl:call-template name="convert-size">
              <xsl:with-param name="size" select="@width" />
              <xsl:with-param name="res-size" select="'800'" />
            </xsl:call-template>
          </xsl:variable>
          <xsl:attribute name="width">
            <xsl:value-of select="$width" />
          </xsl:attribute>
        </xsl:if>
        <xsl:if test="@height!=''">
          <xsl:variable name="height">
            <xsl:call-template name="convert-size">
              <xsl:with-param name="size" select="@height" />
              <xsl:with-param name="res-size" select="'800'" />
            </xsl:call-template>
          </xsl:variable>
          <xsl:attribute name="height">
            <xsl:value-of select="$height" />
          </xsl:attribute>
        </xsl:if>
      </img>
    </xsl:if>
  </xsl:template>
  <xsl:template name="convert-size">
    <xsl:param name="size" />
    <xsl:param name="res-size" />
    <xsl:choose>
      <xsl:when test="contains($size, '%')">
        <xsl:value-of select="$res-size*($size/100)" />
      </xsl:when>
    </xsl:choose>
  </xsl:template>



  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <!-- This was a solution given to me from the xsl-mailing
       list... hear's some of what he said:
       
              This is pretty heavy stuff. If I had the choice, I'd get
              the design of the XML changed up front so this wouldn't be
              necessary. The design of XSLT 1.0 reflects a philosophy
              that the structure of the result should reflect the
              structure of the source more or less transparently:
              strictly speaking you'd have to call this an
              up-conversion, not what the language was designed for.

       Maybe a design change to make this more structured? What he's
       saying (as far as I can tell) is that since attributes are by
       definition not in any order, when you have a label, when
       converted to HTML needs to have attributes converted to elements
       in a certain nested order, you're converting to something more
       structured rather than equally or less structured. 

       In any case, it works and it's fast. -->
  <!--- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <xsl:template match="label">
    <!-- <xsl:if test="ancestor-or-self::*/@width!=''">
           <xsl:attribute name="width">
             <xsl:value-of select="@width" />
           </xsl:attribute>
         </xsl:if> -->
    <xsl:call-template name="check-paragraph">
      <xsl:with-param name="contents">
        <xsl:call-template name="atts-to-elems">
          <xsl:with-param name="attributes" select="ancestor-or-self::*/@*[not(local-name()='paragraph')]"/>
          <xsl:with-param name="contents">
            <xsl:apply-templates/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="atts-to-elems">
    <xsl:param name="contents"/>
    <xsl:param name="attributes" select="false()"/>
    <xsl:choose>
      <xsl:when test="$attributes">
        <xsl:apply-templates select="$attributes[1]">
          <xsl:with-param name="contents">
            <xsl:call-template name="atts-to-elems">
              <xsl:with-param name="attributes" select="$attributes[position() != 1]"/>
              <xsl:with-param name="contents" select="$contents"/>
            </xsl:call-template>
          </xsl:with-param>
        </xsl:apply-templates>
      </xsl:when>
      <xsl:otherwise>
        <xsl:copy-of select="$contents"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- <label align="left" paragraph="true" italic="true" bold="true">This is a label</label> -->

  <xsl:template name="check-paragraph">
    <xsl:param name="contents"/>
    <xsl:choose>
      <xsl:when test="@paragraph='true'">
        <p><xsl:copy-of select="$contents"/></p>
      </xsl:when>
      <xsl:otherwise>
        <xsl:copy-of select="$contents"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="@italic[.='true']">
    <xsl:param name="contents"/>
    <i><xsl:copy-of select="$contents"/></i>
  </xsl:template>

  <xsl:template match="@bold[.='true']">
    <xsl:param name="contents"/>
    <b><xsl:copy-of select="$contents"/></b>
  </xsl:template>


  <xsl:template match="@*">
    <xsl:param name="contents"/>
    <xsl:copy-of select="$contents"/>
  </xsl:template>
  
  <xsl:template match="label/text()">
    <xsl:call-template name="break" />
  </xsl:template>
    
  <xsl:template name="break">
    <xsl:param name="text" select="."/>
    <xsl:choose>
      <xsl:when test="contains($text, '&#xa;')">
        <xsl:value-of select="substring-before($text, '&#xa;')"/>
        <br/>
        <xsl:call-template name="break">
          <xsl:with-param name="text" select="substring-after($text,'&#xa;')"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
</xsl:stylesheet>
