#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "debug.h"
#include "xml_funcs.h"
#include "xml_display.h"
#include "html_output.h"

static int process_label (xmlNode *report, FILE *output, xmlNode *label)
{
	return html_label(output, label);
/*
	xmlNode *textnode;
	int isbold, isitalic;
	char *text;

	textnode = label->children;
	text = "";
	if (textnode)
		if (textnode->content)
			text = textnode->content;
	
	isitalic = xml_node_parent_attr_val (label, "italic", "true");
	isbold = xml_node_parent_attr_val (label, "bold", "true");

	printf ("%s%s%s%s%s\n", isitalic ? "<i>" : "", isbold ? "<b>" : "", text,
				isbold ? "</b>" : "", isitalic ? "</i>" : "");
	
	return 0;
*/
}

static int process_cell (xmlNode *report, FILE *output, xmlNode *cell)
{
	xmlNode *item;

/*	printf ("Processing cell: %s\n", cell->name); */

	output_cell_start (output);

	for (item = cell->children; item != NULL; item = item->next) {
		if (!strcmp (item->name, "label")) {
			html_label (output, item);
		}
		/*
		else if (!strcmp (item->name, "text")) {
			html_string (output, item);
		}
		*/
		else if (!strcmp (item->name, "image")) {
			html_image(output, item);
		}
		else
			errormsg ("Invalid cell item: %s", item->name);
	}

	output_cell_end (output);

	return 0;
}

static int process_row (xmlNode *report, FILE *output, xmlNode *row)
{
	xmlNode *cell;

/*	printf ("Processing row: %s\n", row->name); */

	output_row_start (output);

	for (cell = row->children; cell != NULL; cell = cell->next)
		{
		if (strcmp (cell->name, "cell") == 0)
			process_cell (report, output, cell);
		else if (strcmp (cell->name, "text") == 0)
			;
		else
			errormsg ("Invalid row item: %s", cell->name);
		}

	output_row_end (output);

	return 0;
}

static int process_pageheader (xmlNode *report, FILE *output, xmlNode *header)
{
	xmlNode *row;

	output_pageheader_start (output);

	for (row = header->children; row != NULL; row = row->next)
		{
		if (strcmp (row->name, "row") == 0)
			process_row (report, output, row);
		else if (strcmp (row->name, "text") == 0)
			;
		else
			errormsg ("Invalid page header item: %s", row->name);
		}

	output_pageheader_end (output);

	return 0;
}

static int process_pagefooter (xmlNode *report, FILE *output, xmlNode *footer)
{
	return 0;
}

static int process_table (xmlNode *report, FILE *output, xmlNode *table)
{
	xmlNode *item;
	char *border;

	/* xml_display_node (table); */

	border = xml_node_attr_val (table, "border");

	fprintf (output, "<table");
	if (border) {
		fprintf (output, " border=\"%s\"", border);
	}
	fprintf (output, ">");

	for (item = table->children; item != NULL; item = item->next) {
		if (strcmp (item->name, "header") == 0) {
			process_pageheader (report, output, item);
		}
		else if (!strcmp (item->name, "footer")) {
			process_pagefooter (report, output, item);
		}
		else if (!strcmp (item->name, "row")) {
			process_row (report, output, item);
		}
		else if (!strcmp (item->name, "cell")) {
			process_cell (report, output, item);
		}
		else if (!strcmp (item->name, "text")) {
			;
		}
		else if (!strcmp (item->name, "line")) {
			;
		}
		else {
			errormsg ("Invalid table item: %s", item->name);
		}
	}

	fprintf (output, "\n</table>\n");

	return 0;
}

static int process_line(xmlNode *report, FILE *output, xmlNode *line)
{
    char *style;
    char *width;

    style = xml_node_attr_val (line, "style");
	/* Set default value */
    style = style ? style : "single";
    width = xml_node_attr_val (line, "width");
	/* Set default value */
    width = width ? width : "100%";

    if (strcmp (style, "single") == 0)
        fprintf (output, "<HR WIDTH=\"%s\"><BR>\n", width);
    else if (strcmp (style, "double") == 0)
        fprintf (output, "<HR WIDTH=\"%s\"><HR WIDTH=\"%s\"><BR>\n",
			width, width);
    else
        errormsg ("Invalid line style: %s", style);

	return 0;
}

static int process_spacer(xmlNode *report, FILE *output, xmlNode *spacer)
{
    char *style;
    char *size = NULL;
    xmlNode *child;

    style = xml_node_attr_val (spacer, "style");
    style = style ? style : "vertical";

	/* Check if syntax are OK */
    for (child = spacer->children; child != NULL; child = child->next)
        {
        if (strcmp (child->name, "text") == 0)
            size = child->content;
        else
            errormsg ("Invalid spacer item: %s", child->name);
        }

    if (size == NULL)
        {
        errormsg ("Spacer node must have content");
        return -1;
        }

    if (!strcmp (style, "vertical"))
	{
		/* I assume the unit of the given legnth is always cm */
		int i;
		int str_size;
		char *num_str;

		str_size  = strlen(size);
		/* Calculate size of number. e.g. "123cm" would be 3 and
		"12345cm" would be 5 */
		for (i = 0; i < str_size && isdigit(size[i]); i++);
		num_str = strndup(size, i);

		for (i = 0; i < atoi(size); i++) {
			fputs("<BR>", output);
		}

		free(num_str);

	}
    else if (!strcmp (style, "horizontal"))
	{
		/* I assume the unit of the given legnth is always cm */
		int i;
		int str_size;
		char *num_str;

		str_size  = strlen(size);
		/* Calculate size of number. e.g. "123cm" would be 3 and
		"12345cm" would be 5 */
		for (i = 0; i < str_size && isdigit(size[i]); i++);
		num_str = strndup(size, i);

		for (i = 0; i < atoi(size); i++) {
			fputs("&nbsp;&nbsp;", output);
		}

		free(num_str);
	}
    else
        errormsg ("Invalid spacer style: %s", style);

	return 0;
}

static int process_newpage()
{
	/* Francis - I dunno what to do :-( */
	return 0;
}


static int process_report (xmlNode *report, FILE *output)
{
	xmlNode *item;
	char *report_name = "Papyrus Report";
	xmlAttr *attr;



	for (attr = report->properties ; attr != NULL; attr = attr->next)
		{
		if (strcmp (attr->name, "name") == 0)
			report_name = attr->children->content;
		}


	output_header (output, report_name);

	for (item = report->children; item != NULL; item = item->next)
		{		
		if (!strcmp (item->name, "table"))
			{
			process_table (report, output, item);
			}
		else if (!strcmp (item->name, "label"))
			{
			process_label (report, output, item);
			}
		else if (!strcmp (item->name, "line"))
			{
			process_line(report, output, item);
			}
		else if (!strcmp (item->name, "spacer"))
			{
			process_spacer(report, output, item);
			}
		else if (!strcmp (item->name, "newpage"))
			{
			process_newpage(report, output, item);
			}

		/* Do nothing for "text" and "comment" */
		else if (!strcmp (item->name, "text"))
			{
			/* fprintf(output, convert_specialchars(item)); */
			}
		else if (!strcmp (item->name, "comment"))
			;
		else if (!strcmp(item->name, "title"));
		else
			{
			errormsg ("Invalid item: %s", item->name);
			}
		}

	output_footer (output, NULL);

	return 0;
}

int xml_doc_to_html(xmlDoc *doc, FILE *output)
{
	xmlNode *report;

	dassert (doc);

    report = doc->children;

    dassert (report);
    dassert (report->name);

    if (strcmp (report->name, "report") != 0)
        {
        errormsg ("Invalid report first item: %s", report->name);
        return -1;
        }

    process_report (report, output);

    return 0;

}
